"""
Copyright 2015,2016 Hermann Krumrey

This file is part of hikvision-client.

    hikvision-client offers a graphical user interface to access Hikvision IP
    cameras over RTSP.

    hikvision-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    hikvision-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with hikvision-client.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
from typing import Tuple, List
from subprocess import Popen, PIPE
from gfworks.templates.generators.GridTemplateGenerator import GridTemplateGenerator
templates = GridTemplateGenerator.get_grid_templates()
try:
    used_template = templates["gtk3"]
except KeyError:
    used_template = templates["tk"]


# noinspection PyUnresolvedReferences,PyAbstractClass
class MainGui(used_template):
    """
    Class that models the GUI
    """

    username = ""
    """
    The user's username
    """

    password = ""
    """
    The user's password
    """

    cameras = []
    """
    The list of cameras read while parsing the config
    """

    camera_buttons = []
    """
    The list of camera buttons
    """

    def __init__(self, credentials: Tuple[str, str], cameras: List[Tuple[str, str]]):
        """
        Constructor, which asks the user for a username and password, and which group of cameras to use
        Afterwards, a selection of cameras is shown that can be accessed

        :param credentials: Tuple of username and password
        :param cameras: The list of cameras to which the user can connect to
        """
        self.username, self.password = credentials
        self.cameras = cameras
        super().__init__("Hikvision Client")

    def lay_out(self) -> None:
        """
        Handles the layout of the GUI

        :return: None
        """

        for camera in self.cameras:
            camera_name, camera_link = camera

            self.camera_buttons.append(self.generate_button(camera_name, self.start_stream, camera_link))

        row = 0
        column = 0
        for camera in self.camera_buttons:
            self.position_absolute(camera, column, row, 2, 1)
            column += 2

            if column % 6 == 0:
                row += 1
                column = 0

    def start_stream(self, widget: object, this_camera_link: str) -> None:
        """
        Starts the video feed for a camera

        :param widget: the button calling this method
        :param this_camera_link: the camera link used
        :return: None
        """
        str(widget)
        args = ["vlc",
                "-vvv",
                "--live",
                "--no-drop-late-frames",
                "--no-skip-frames",
                "--rtsp-tcp",
                ("rtsp://" + self.username + ":" + self.password + "@" + this_camera_link).rstrip()]
        Popen(args, stderr=PIPE).wait()



"""
Copyright 2015,2016 Hermann Krumrey

This file is part of hikvision-client.

    hikvision-client offers a graphical user interface to access Hikvision IP
    cameras over RTSP.

    hikvision-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    hikvision-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with hikvision-client.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import sys
from typing import List, Tuple
from hikvision_client.userinterface.MainGui import MainGui
from gfworks.templates.generators.GridTemplateGenerator import GridTemplateGenerator
templates = GridTemplateGenerator.get_grid_templates()
try:
    used_template = templates[sys.argv[1]]
except KeyError:
    used_template = templates["tk"]


# noinspection PyAbstractClass,PyUnresolvedReferences
class LoginGui(used_template):
    """
    A Login Screen for the program
    """

    cameras = []
    """
    The list of cameras read from the config file
    """

    login_button = None
    """
    The login button
    """

    username_label = None
    """
    A Label that displays the string "Username"
    """

    username_entry = None
    """
    An entry widget for the user to enter his/her username
    """

    password_label = None
    """
    A Label that displays the string "Password"
    """

    password_entry = None
    """
    A password entry widget for the user to enter his/her password
    """

    def __init__(self, cameras: List[Tuple[str, str]]) -> None:
        """
        Calls the constructor of the parent class with the window title "Login"

        :return: None
        """
        super().__init__("Login")
        self.cameras = cameras

    def lay_out(self) -> None:
        """
        Lays out the UI elements

        :return: None
        """
        self.login_button = self.generate_button("Log In", self.login)
        self.username_label = self.generate_label("Username")
        self.username_entry = self.generate_text_entry("")
        self.password_label = self.generate_label("Password")
        self.password_entry = self.generate_password_entry()

        self.position_absolute(self.username_label, 0, 0, 2, 1)
        self.position_absolute(self.password_label, 2, 0, 2, 1)
        self.position_absolute(self.username_entry, 0, 1, 2, 1)
        self.position_absolute(self.password_entry, 2, 1, 2, 1)
        self.position_absolute(self.login_button, 1, 3, 2, 1)

    def login(self, widget: object) -> None:
        """
        Tries to log in. Continues on to the Main Gui on success or shows an error dialog on failure

        :param widget: The button that called this method
        :return: None
        """
        str(widget)
        username = self.get_string_from_text_entry(self.username_entry)
        password = self.get_string_from_text_entry(self.password_entry)

        if username and password:
            self.stop()
            MainGui((username, password), self.cameras).start()
        else:
            self.show_message_dialog("Error Logging In", "Please enter both a username and a password")



"""
Copyright 2015,2016 Hermann Krumrey

This file is part of hikvision-client.

    hikvision-client offers a graphical user interface to access Hikvision IP
    cameras over RTSP.

    hikvision-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    hikvision-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with hikvision-client.  If not, see <http://www.gnu.org/licenses/>.
"""

"""
The metadata is stored here. It can be used by any other module in this project this way, most
notably by the setup.py file
"""

project_name = "hikvision_client"
"""
The name of the project
"""

project_description = "A graphical client software for Hikvision (and other RTSP-enabled) IP cameras"
"""
A short description of the project
"""

version_number = "0.1"
"""
The current version of the program.
"""

development_status = "Development Status :: 2 - Pre-Alpha"
"""
The current development status of the program
"""

audience = "Intended Audience :: End Users/Desktop"
"""
The intended audience of this software
"""

environment = "Environment :: Other Environment"
"""
The intended environment in which the program will be used
"""

programming_language = 'Programming Language :: Python :: 3'
"""
The programming language used in this project
"""

topic = "Topic :: Multimedia :: Video :: Display"
"""
The broad subject/topic of the project
"""

language = "Natural Language :: English"
"""
The (default) language of this project
"""

compatible_os = "Operating System :: OS Independent"
"""
Th eOperating Systems on which the program can run
"""

project_url = "http://namibsun.net/namboy94/hikvision-client"
"""
A URL linking to the home page of the project, in this case a
self-hosted Gitlab page
"""

download_url = "http://gitlab.namibsun.net/namboy94/hikvision-client/repository/archive.zip?ref=master"
"""
A URL linking to the current source zip file.
"""

author_name = "Hermann Krumrey"
"""
The name(s) of the project author(s)
"""

author_email = "hermann@krumreyh.com"
"""
The email address(es) of the project author(s)
"""

license_type = "GNU GPL3"
"""
The project's license type
"""

license_identifier = "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
"""
The license used for this project
"""

pypi_requirements = []
"""
List of requirements available on pypi.
"""

other_requirements = []
"""
List of requirements not available on pypi
"""

scripts = []
"""
List of script files to be installed during installation
"""



"""
Copyright 2015,2016 Hermann Krumrey

This file is part of hikvision-client.

    hikvision-client offers a graphical user interface to access Hikvision IP
    cameras over RTSP.

    hikvision-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    hikvision-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with hikvision-client.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import hikvision_client.metadata as metadata
from setuptools import setup, find_packages


def readme() -> str:
    """
    Reads the readme file and converts it from markdown to restructured text

    :return: the readme file as a string
    """
    try:
        # noinspection PyPackageRequirements
        import pypandoc
        with open('README.md') as f:
            # Convert markdown file to rst
            markdown = f.read()
            rst = pypandoc.convert(markdown, 'rst', format='md')
            return rst
    except (OSError, ImportError):
        # If pandoc is not installed, just return the raw markdown text
        with open('README.md') as f:
            return f.read()

setup(name=metadata.project_name,
      version=metadata.version_number,
      description=metadata.project_description,
      long_description=readme(),
      classifiers=[metadata.development_status,
                   metadata.license_identifier,
                   metadata.audience,
                   metadata.programming_language,
                   metadata.topic,
                   metadata.language,
                   metadata.compatible_os,
                   metadata.environment],
      url=metadata.project_url,
      author=metadata.author_name,
      author_email=metadata.author_email,
      license=metadata.license_type,
      packages=find_packages(),
      install_requires=metadata.pypi_requirements,
      requires=metadata.other_requirements,
      scripts=metadata.scripts,
      test_suite='nose.collector',
      tests_require=['nose'],
      zip_safe=False)


"""
Copyright 2015,2016 Hermann Krumrey

This file is part of hikvision-client.

    hikvision-client offers a graphical user interface to access Hikvision IP
    cameras over RTSP.

    hikvision-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    hikvision-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with hikvision-client.  If not, see <http://www.gnu.org/licenses/>.
"""

from hikvision_client.main import main


if __name__ == '__main__':
    main("tk")



"""
Copyright 2015,2016 Hermann Krumrey

This file is part of hikvision-client.

    hikvision-client offers a graphical user interface to access Hikvision IP
    cameras over RTSP.

    hikvision-client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    hikvision-client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with hikvision-client.  If not, see <http://www.gnu.org/licenses/>.
"""

# imports
import os
from typing import List, Tuple


class ConfigParser(object):
    """
    Class that parses camera config files
    """

    @staticmethod
    def parse_camera_config() -> List[Tuple[str, str]]:
        """
        Parses a configuration file to load the user's configured cameras
        
        :return: a list of tuples consisting of the name and address of a camera
        """
        config_file = os.path.join(os.path.expanduser("~"), ".hikvision-client", "cam_config")

        if not os.path.isdir(os.path.join((os.path.expanduser("~"), ".hikvision-client"))):
            os.makedirs(os.path.join((os.path.expanduser("~"), ".hikvision-client")))
        if not os.path.isfile(config_file):
            open(config_file, 'w').close()

        with open(config_file, 'r') as config:
            content = config.read().split("\n")
            cameras = []

            for line in content:
                if line:
                    cam_name, cam_link = line.split("#####")
                    cameras.append((cam_name, cam_link))

        return cameras
